package com.aitrainer.api.controller

import com.aitrainer.api.model.*
import com.aitrainer.api.model.CustomerMembership
import com.aitrainer.api.model.diet.*
import com.aitrainer.api.repository.*
import com.aitrainer.api.repository.diet.*
import com.google.gson.GsonBuilder
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/api")
class CustomerPackageController( private val customerRepository: CustomerRepository,
                                 private val customerExerciseDeviceRepository: CustomerExerciseDeviceRepository,
                                 private val exercisesRepository: ExercisesRepository,
                                 private val purchaseRepository: PurchaseRepository,
                                 private val customerPropertyRepository: CustomerPropertyRepository,
                                 private val exerciseResultRepository: ExerciseResultRepository,
                                 private val customerActivityRepository: CustomerActivityRepository,
                                 private val customerTrainingPlanRepository: CustomerTrainingPlanRepository,
                                 private val customerMembership: CustomerMembershipRepository,
                                    private val dietRepository: DietRepository,
                                    private val dietUserConsumptionRepository: DietUserConsumptionRepository,
                                    private val dietUserRepository: DietUserRepository,
                                    private val dietUserPreferenceRepository: DietUserPreferenceRepository,
                                    private val dietUserSensitivityRepository: DietUserSensitivityRepository,
                                    private val customerConversationRepository: CustomerConversationRepository

) {

    @GetMapping("/diet_customer_package/{id}")
    fun getDietCustomerPackageData(@PathVariable(value = "id") dietUserId: Long): ResponseEntity<String> {
        if (dietUserId <= 0) {
            return ResponseEntity.notFound().build()
        }

        val gson = GsonBuilder()
            .excludeFieldsWithoutExposeAnnotation()
            .setPrettyPrinting()
            .create()

        val dietUser: DietUser = dietUserRepository.findByDietUserId(dietUserId)
            ?: return ResponseEntity.notFound().build()

        val customerId = dietUser.customerId
        val customer: Customer = customerRepository.findByCustomerIdAndActive(customerId, "Y")
            ?: return ResponseEntity.notFound().build()

        val customerJson: String = gson.toJson(customer)
        val dietUserJson: String = gson.toJson(dietUser)

        val listDiet = dietRepository.findByDietUserId(dietUserId)
        val listDietJson = gson.toJson(listDiet)

        val listDietUserConsumption = dietUserConsumptionRepository.findByDietUserId(dietUserId)
        val listDietUserConsumptionJson = gson.toJson(listDietUserConsumption)

        val listDietUserPreference = dietUserPreferenceRepository.findByDietUserId(dietUserId)
        val listDietUserPreferenceJson = gson.toJson(listDietUserPreference)

        val listDietUserSensitivity = dietUserSensitivityRepository.findByDietUserId(dietUserId)
        val listDietUserSensitivityJson = gson.toJson(listDietUserSensitivity)

        val listCustomerConversation = customerConversationRepository.findByCustomerId(customerId)
        val listCustomerConversationJson = gson.toJson(listCustomerConversation)

        val packageJson: String =
                             getClassRecord(Customer::class.simpleName, customerJson) +
                    "|||" +  getClassRecord(DietUser::class.simpleName, dietUserJson) +
                    "|||" +  getClassRecord(Diet::class.simpleName, listDietJson) +
                    "|||" +  getClassRecord(DietUserConsumption::class.simpleName, listDietUserConsumptionJson) +
                    "|||" +  getClassRecord(DietUserPreference::class.simpleName, listDietUserPreferenceJson) +
                    "|||" +  getClassRecord(DietUserSensitivity::class.simpleName, listDietUserSensitivityJson) +
                    "|||" +  getClassRecord(CustomerConversation::class.simpleName, listCustomerConversationJson)

        return if (packageJson.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(packageJson)
    }

    @GetMapping("/club_customer_package/{id}")
    fun getCustomerClubPackageData(@PathVariable(value = "id") customerId: Long): ResponseEntity<String> {
        if (customerId <= 0) {
            return ResponseEntity.notFound().build()
        }

        val gson = GsonBuilder()
            .excludeFieldsWithoutExposeAnnotation()
            .setPrettyPrinting()
            .create()

        val customer: Customer = customerRepository.findByCustomerIdAndActive(customerId, "Y")
            ?: return ResponseEntity.notFound().build()

        val customerJson: String = gson.toJson(customer)

        val listCustomerPropertyAll = customerPropertyRepository.findAllByCustomerId(customerId)
        val listCustomerPropertyAllJson = gson.toJson(listCustomerPropertyAll)
        val listCustomerProperty = customerPropertyRepository.findLastPropertiesByCustomerId(customerId)
        val listCustomerPropertyJson = gson.toJson(listCustomerProperty)

        val listTrainingPlan = customerTrainingPlanRepository.findAllByCustomerId(customerId)
        val listTrainingPlanJson = gson.toJson(listTrainingPlan)

        val listMembership = customerMembership.findAllByCustomer(customer)
        val listMembershipJson = gson.toJson(listMembership)

        val packageJson: String =
            getClassRecord(Customer::class.simpleName, customerJson) +
                    "|||" +  getClassRecord(CustomerProperty::class.simpleName+"All", listCustomerPropertyAllJson) +
                    "|||" +  getClassRecord(CustomerProperty::class.simpleName, listCustomerPropertyJson) +
                    "|||" +  getClassRecord(CustomerTrainingPlan::class.simpleName, listTrainingPlanJson +
                    "|||" + getClassRecord(CustomerMembership::class.simpleName, listMembershipJson)
                    )

        return if (packageJson.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(packageJson)
    }

    @GetMapping("/app_customer_package/{id}")
    fun getCustomerPackageData(@PathVariable(value = "id") customerId: Long): ResponseEntity<String> {

        if (customerId <= 0) {
            return ResponseEntity.notFound().build()
        }

        val gson = GsonBuilder()
            .excludeFieldsWithoutExposeAnnotation()
            .setPrettyPrinting()
            .create()

        val customer: Customer = customerRepository.findByCustomerIdAndActive(customerId, "Y")
            ?: return ResponseEntity.notFound().build()

        val customerJson: String = gson.toJson(customer)

        val listCustomerExerciseDevices = customerExerciseDeviceRepository.findByCustomerId(customerId)
        val listCustomerExerciseDeviceJson = gson.toJson(listCustomerExerciseDevices)

        val listExercises = exercisesRepository.getAllByCustomerIdOrderByDateAddDesc(customerId)
        val listExercisesJson = gson.toJson(listExercises)

        val listPurchase = purchaseRepository.findByCustomerId(customerId)
        val listPurchaseJson = gson.toJson(listPurchase)

        val listCustomerPropertyAll = customerPropertyRepository.findAllByCustomerId(customerId)
        val listCustomerPropertyAllJson = gson.toJson(listCustomerPropertyAll)
        val listCustomerProperty = customerPropertyRepository.findLastPropertiesByCustomerId(customerId)
        val listCustomerPropertyJson = gson.toJson(listCustomerProperty)


        val listExerciseResult = exerciseResultRepository.getAllByCustomerId(customerId)
        val listExerciseResultJson = gson.toJson(listExerciseResult)

        val listActivityResult = customerActivityRepository.findByCustomerId(customerId)
        val listActivityJson = gson.toJson(listActivityResult)

        val listTrainingPlan = customerTrainingPlanRepository.findAllByCustomerId(customerId)
        val listTrainingPlanJson = gson.toJson(listTrainingPlan)

        val packageJson: String =
            getClassRecord(Customer::class.simpleName, customerJson) +
                    "|||" +  getClassRecord(CustomerExerciseDevice::class.simpleName, listCustomerExerciseDeviceJson) +
                    "|||" +  getClassRecord(Exercises::class.simpleName, listExercisesJson) +
                    "|||" +  getClassRecord(Purchase::class.simpleName, listPurchaseJson) +
                    "|||" +  getClassRecord(CustomerProperty::class.simpleName+"All", listCustomerPropertyAllJson) +
                    "|||" +  getClassRecord(CustomerProperty::class.simpleName, listCustomerPropertyJson) +
                    "|||" +  getClassRecord(ExerciseResult::class.simpleName, listExerciseResultJson+
                    "|||" +  getClassRecord(CustomerActivity::class.simpleName, listActivityJson) +
                    "|||" +  getClassRecord(CustomerTrainingPlan::class.simpleName, listTrainingPlanJson))

                    return if (packageJson.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(packageJson)
    }

    fun getClassRecord(className: String?, json: String): String {
        return "$className***$json"
    }
}