package com.aitrainer.api.test

import com.aitrainer.api.controller.CustomerTrainingPlanExerciseController
import com.aitrainer.api.model.CustomerTrainingPlanExercise
import com.aitrainer.api.repository.CustomerTrainingPlanExerciseRepository
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.HttpStatus
import kotlin.test.assertEquals
import kotlin.test.assertTrue

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class CustomerTrainingPlanExerciseTest {

    @Autowired
    private lateinit var customerTrainingPlanExerciseRepository: CustomerTrainingPlanExerciseRepository



    @Test
    fun testInsert() {
        val customerTrainingPlanExercise = CustomerTrainingPlanExercise(
            customerId = 90,
            exerciseId = 60,
            repeats = 20,
            weight = 50.0,
            customerTrainingPlanDetailsId = 1
        )

        val controller = CustomerTrainingPlanExerciseController(customerTrainingPlanExerciseRepository)
        val response = controller.createNewCustomerTrainingPlanExercise(customerTrainingPlanExercise)

        val newCustomerTrainingPlanExercise = response.body

        assertTrue(newCustomerTrainingPlanExercise is CustomerTrainingPlanExercise)
        assertEquals(newCustomerTrainingPlanExercise.exerciseId, 60)
        assertEquals(newCustomerTrainingPlanExercise.repeats, 20)
        assertEquals(newCustomerTrainingPlanExercise.weight, 50.0)

        customerTrainingPlanExerciseRepository.delete(newCustomerTrainingPlanExercise)
    }

    @Test
    fun testGet() {
        val controller = CustomerTrainingPlanExerciseController(customerTrainingPlanExerciseRepository)
        val response = controller.getAllByCustomerId(90)

        assertEquals(response.statusCode, HttpStatus.OK)
        val exerciseList = response.body

        assertEquals(exerciseList!!.size, 3)
        assertEquals(exerciseList[1].weight, 26.0)
        assertEquals(exerciseList[1].repeats, 14)
        assertEquals(exerciseList[2].weight, 30.0)
        assertEquals(exerciseList[2].repeats, 11)


    }


}