package com.aitrainer.api.test.diet

import com.aitrainer.api.model.diet.Meal
import com.aitrainer.api.repository.diet.MealRepository
import com.aitrainer.api.test.Tokenizer
import com.google.gson.Gson
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.junit.jupiter.SpringExtension
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.*


@ExtendWith(SpringExtension::class)
@SpringBootTest
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class MealTest {

    @Autowired
    private lateinit var mockMvc: MockMvc

    @Autowired
    private lateinit var mealRepository: MealRepository

    private val meal = Meal(

        name = "Tükörtojás",
        quantity = 90.0,
        quantityUnit = "g",
        calMin = 70.0,
        calMax = 110.0,
        proteinMin = 19.0,
        proteinMax = 25.0,
        fatMin = 12.0,
        fatMax = 15.0,
        chMin = 1.0,
        chMax = 3.0,
        sugar = 0.0,

    )
    private val meal2 = Meal(

        name = "Tigris buci",
        quantity = 3.0,
        quantityUnit = "db",
        calMin = 123.0,
        calMax = 140.0,
        proteinMin = 19.0,
        proteinMax = 25.0,
        fatMin = 30.0,
        fatMax = 34.0,
        chMin = 50.0,
        chMax = 60.0,
        sugar = 5.0,
    )
    private val meal3 = Meal(

        name = "Töltötttojás",
        quantity = 330.0,
        calMin = 75.0,
        calMax = 112.0,
        proteinMin = 12.0,
        proteinMax = 20.0,
        fatMin = 12.0,
        fatMax = 15.0,
        chMin = 1.0,
        chMax = 3.0,
        sugar = 0.0,

    )

    private var authToken: String? = ""

    @BeforeAll
    fun initTest() {
        authToken = Tokenizer.getToken()
    }


    @Test
    fun `insert meals successfully`() {

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/meal")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(meal))
        )
        .andExpect(status().isOk)
        .andExpect(jsonPath("$.sugar").value(0.0))
        .andExpect(jsonPath("$.quantity").value(90.0))
        .andExpect(jsonPath("$.proteinMax").value(25.0))

        var mvcResult = mockMvc.perform(
            MockMvcRequestBuilders.post("/api/meal")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(meal2))
        )
            .andExpect(status().isOk)
            .andReturn()

        val gson= Gson()
        val newMealJson = mvcResult.response.contentAsString
        val newMeal = gson.fromJson(newMealJson, Meal::class.java)

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/meal")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(meal3))
        )
            .andExpect(status().isOk)

         // Act & Assert
        mockMvc.perform(get("/api/meal" )
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.[3].name").value("Tükörtojás"))
            .andExpect(jsonPath("$.[4].name").value("Tigris buci"))
            .andExpect(jsonPath("$.[5].quantity").value(330.0))
            .andExpect(jsonPath("$.[5].name").value("Töltötttojás"))


        // Act & Assert
        mockMvc.perform(get("/api/meal/${newMeal.id}")
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.name").value("Tigris buci"))

        mockMvc.perform(get("/api/meal/by_name/Töltötttojás")
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.name").value("Töltötttojás"))
            .andExpect(jsonPath("$.quantity").value(330.0))


        mvcResult = mockMvc.perform(get("/api/meal/${newMeal.id-1}")
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            . andReturn()

        val newMealJson1 = mvcResult.response.contentAsString
        val newMeal1 = gson.fromJson(newMealJson1, Meal::class.java)

        mealRepository.delete(newMeal1)
        mealRepository.delete(newMeal)

        mvcResult = mockMvc.perform(get("/api/meal/${newMeal.id+1}")
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            . andReturn()

        val newMealJson3 = mvcResult.response.contentAsString
        val newMeal3 = gson.fromJson(newMealJson3, Meal::class.java)

        mealRepository.delete(newMeal3)
    }

    private fun toJson(obj: Any): String {
        return Gson().toJson(obj)
    }
}
